#ifndef MT_UCISP_SDK_H
#define MT_UCISP_SDK_H

typedef int SDK_RETURN_CODE;
typedef unsigned int DEV_HANDLE;

#ifdef SDK_EXPORTS
#define SDK_API extern "C" __declspec(dllexport) SDK_RETURN_CODE _cdecl
#define SDK_HANDLE_API extern "C" __declspec(dllexport) DEV_HANDLE _cdecl
#define SDK_POINTER_API extern "C" __declspec(dllexport) unsigned char * _cdecl
#else
#define SDK_API extern "C" __declspec(dllimport) SDK_RETURN_CODE _cdecl
#define SDK_HANDLE_API extern "C" __declspec(dllimport) DEV_HANDLE _cdecl
#define SDK_POINTER_API extern "C" __declspec(dllimport) unsigned char *_cdecl
#endif

typedef struct
{
  int  CameraID,
  int  Column,
  int  AvgFrameCnt,
  int  ExposureTime,
  int  GreenGain,
  int  TimeStamp,
  int  OpticalBlack,
  int  FrameTime,
  int  SpectroNum;

}TSpectrumDataProperty;

typedef void(*SpectrometerFrameCallback)(TSpectrumDataProperty *Property, double **FloatPtr );

SDK_API MTUCISP_InitDevice( void );
SDK_API MTUCISP_AddDeviceToWorkingSet(int DeviceID);
SDK_API MTUCISP_RemoveDeviceFromWorkingSet(int deviceID);
SDK_API MTUCISP_InstallSpectrometerFrameHooker(int deviceID,SpectrometerFrameCallback HookerProc);
SDK_API MTUCISP_StartDeviceEngine(void);
SDK_API MTUCISP_StartFrameGrab(int GrabType);
SDK_API MTUCISP_StopFrameGrab(void);
SDK_API MTUCISP_StopDeviceEngine(void);
SDK_API MTUCISP_UnInitDevice(void);

SDK_API MTUCISP_GetDeviceModuleNoSerialNo(int DeviceId, char *ModuleNo, char *SerialNo);
SDK_API MTUCISP_GetDeviceCCDSize(int DeviceID,int *CCDSize);
SDK_API MTUCISP_GetDeviceSpectrometerNum(int DeviceID);

SDK_API MTUCISP_SetDeviceWorkMode(int deviceID,int WorkMode);
SDK_API MTUCISP_SetDeviceExposureTime(int deviceID,int ExpTime);
SDK_API MTUCISP_SetDeviceCCDTotalGain(int DeviceID,int Gain);
SDK_API MTUCISP_SetDeviceCCDOpticBlack(int eviceId,int OBValue);
SDK_API MTUCISP_SetDeviceAverageFrameNum(int deviceID,int AvgFrameNum);

SDK_API MTUCISP_GetDeviceSpectrometerETCStatus(int deviceID,int SpectroID);
SDK_API MTUCISP_SetDeviceSpectrometerETCStatus(int deviceID,int SpectroID,int flag);

SDK_API MTUCISP_GetDeviceSpectrometerDisplayData(int DeviceID,int spectroID, int WaitFlag, double* &Data);
SDK_API MTUCISP_SetDeviceSpectrometerDarkData(int DeviceID,int spectroID,double *Darkata);
SDK_API MTUCISP_SetDeviceSpectrometerRefData(int DeviceID,int spectroID,double *RefData);
SDK_API MTUCISP_GetDeviceSpectrometerWavValue(int DeviceID,int SpectroID,int PixelNo);

SDK_API MTUCISP_GetDeviceSpectrometerSpectrum(int DeviceID,int SpectroID,double *Input,double *Output);
SDK_API MTUCISP_GetDeviceSpectrometerCIE1931Coords(int DeviceID,int spectroID,double *Input, double &x, double &y); 
SDK_API MTUCISP_GetDeviceSpectrometerCIE1976Coords(int DeviceID,int spectroID,double *Input, double &up, double &vp); 
SDK_API MTUCISP_GetDeviceSpectrometerCCT(int DeviceID,int spectroID,double *Intput, int &CCT);
SDK_API MTUCISP_GetDeviceSpectrometerCRIs(int DeviceID,int SpectroID, double *Input, double *CRIs);

#endif //#ifdef