unit ShowForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, RzPanel, jpeg , Math, myDataTypes, myDllInterface, mmsystem, Menus, RzStatus;

const
  WM_USER_PAINT = WM_USER + 101;

type

  TPaintInfo = Record
    HasPaintCmd : Boolean;
    ImageRect : TRect;
    BmpPointer : pBitmap;
  end;

  TScrollCtl = Record
    H_AllRange : Integer;
    V_AllRange : Integer;
  end;

  TImageForm = class(TForm)
    ImagePanel: TRzPanel;
    ShowImage: TImage;
    procedure ShowImageMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ShowImageMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure ShowImageMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormCanResize(Sender: TObject; var NewWidth,
      NewHeight: Integer; var Resize: Boolean);
    procedure FormShow(Sender: TObject);
    procedure ShowImageDblClick(Sender: TObject);
    procedure FormHide(Sender: TObject);
  private
    { Private declarations }
//    DeviceType : eDEVICETYPE;
    StartX, StartY : integer;
    DeltaX, DeltaY : integer;
    XPos, XPosMin, XPosMax : integer;
    YPos, YPosMin, YPosMax : integer;
    startTimeTick, endTimeTick : integer;
    frameRate : integer;

    Fitbmp : TBitMap;
    SavedClientRect : TRect;
    AccumulateFrames : integer;
    PaintInfo : TPaintInfo;

//    procedure SetAutoExposureMark( setbmp : TBitMap );
//    procedure SetCCDSpectrumAreaMark(setbmp: TBitMap);

  public

    AlwaysFitWindow : Boolean;
    ShowAutoEXMark : Boolean;
    ControlReSize : Boolean;
    ScrollAllow : Boolean;
    ShowVideo : Boolean;

    { Public declarations }
    procedure WMEraseBkgnd(var m: TWMEraseBkgnd); message WM_ERASEBKGND;
    procedure PaintImageFitWindow( var Msg : TMessage ); message WM_USER_PAINT;    
    procedure FormSizeChanged();
    procedure SetDeviceType( ImageSize:TImageSize );
    procedure SetShowMode( IsFitInWindow : Boolean; IsShowAutoExMark : Boolean );
    procedure DisplayImage( localBitMap : pBitMap );
//    procedure SetStatusPaneInfo( ImageProperty : PProcessedDataProperty ;
//                                 IsSelectedCamera : Boolean );
  end;

var
  NeedReStartFrameRateCaculation : Boolean;
  ImageForm: TImageForm;  // But not created.
implementation

//uses BUFCamera_App, CameraFrame;

{$R *.DFM}
//{$DEFINE LTSHOW}
//{$DEFINE CCDSPECTRUM}


const
  SHOW_THRESHOLD = 1;
  AUTO_EXPOSURE_ROI_WIDTH = 64;
  BRIGHT_LEVEL = 20;
  CAL_FRAMES = 20;

{
  WND: as the CCD camera is extended for spectrum test,
       The spectrum test area has been defined in UnitCCDSpectrum,
       Here we will show the CCD spectrum test area according to the
       visible flag:@flagCSArea.
}
{$IFDEF CCDSPECTRUM}
{
procedure TImageForm.SetCCDSpectrumAreaMark(setbmp: TBitMap);
var
  I: Integer;
  p : PByteArray;
  row, col, startrow, startcol : integer;
  RectWidth : integer;
  CcdAreaWidth: integer;
  SliceIndex :integer;
begin
  if setbmp.Height > AUTO_EXPOSURE_ROI_WIDTH  then // Ignore those resolutions less then ROI
    begin
      for SliceIndex := 0 to MAX_SLICE_CNT-1 do    // Iterate
      begin
        if CSSliceSelectted[SliceIndex] = false then
          Continue;

        StartROW :=  Round(setbmp.Height * CSAreaStartY[SliceIndex] / 100);
        CCdAreaWidth := Round(setbmp.Height  * CSAreaLength[SliceIndex] / 100);
        if CcdAreaWidth <=0 then
          CcdAreaWidth := 1;
        for row := 0 to CCdAreaWidth-1 do
          begin
            p := setbmp.ScanLine[ startrow + row ];
            if setbmp.PixelFormat = pf8bit then // Mono Camera
              // WD: pf8bit - 256 color, for mono color use pf1bit.
              begin
                for col:=0 to setbmp.Width-1 do
                  begin
                    p[col] := Min( 255, p[col] + BRIGHT_LEVEL );
                  end;
              end
            else  // Color Camera
            //WD: 3 byte	- pf24bit?
              begin
                for col:=0 to setbmp.Width-1 do
                  begin
                    p[3*col] := Min( 255, p[3*col] + BRIGHT_LEVEL );
                    p[3*col+1] := Min( 255, p[3*col+1] + BRIGHT_LEVEL );
                    p[3*col+2] := Min( 255, p[3*col+2] + BRIGHT_LEVEL );
                  end;
              end;
          end;
      end;    // sliceLoop

    end;
end;
}
{$ENDIF}
(*
procedure TImageForm.SetAutoExposureMark( setbmp : TBitMap );
var
  p : PByteArray;
  row, col, startrow, startcol : integer;
  {$IFDEF LTSHOW}
  RectWidth : integer;
  {$ENDIF}
  {$IFDEF CCDSPECTRUM}
  CcdAreaWidth: integer;
  {$ENDIF}
begin
  if setbmp.Height > AUTO_EXPOSURE_ROI_WIDTH  then // Ignore those resolutions less then ROI
    begin
      // WD: so the Auto_Exposure_Mark area will always be in the center.
      {$IFDEF LTSHOW}
      StartROW := setbmp.Height * MarkStartY div 100;   // startrow depends on starty while startcol depends on startx
      StartCOL := setbmp.Width  * markstartX div 100;
      rectWidth := setbmp.Height * MarkWidth div 100;
	    if rectWidth <= 0 then
		    rectWidth := 1;
      {$ELSE}
      StartROW := ( setbmp.Height div 2 ) - ( AUTO_EXPOSURE_ROI_WIDTH div 2 );
      StartCOL := ( setbmp.Width div 2 ) - ( AUTO_EXPOSURE_ROI_WIDTH div 2 );
      {$ENDIF}
      {$IFDEF LTSHOW}
      for row := 0 to rectWidth-1 do
      {$ELSE}
      for row := 0 to AUTO_EXPOSURE_ROI_WIDTH-1 do
      {$ENDIF}
        begin
          p := setbmp.ScanLine[ startrow + row ];
          if setbmp.PixelFormat = pf8bit then // Mono Camera
		  // WD: pf8bit - 256 color, for mono color use pf1bit.
            begin
              {$IFDEF LTSHOW}
              for col:=startcol to startcol+rectWidth-1 do
              {$ELSE}
              for col:=startcol to startcol+AUTO_EXPOSURE_ROI_WIDTH-1 do
              {$ENDIF}
                begin
                  p[col] := Min( 255, p[col] + BRIGHT_LEVEL );
                end;
            end
          else  // Color Camera
          //WD: 3 byte	- pf24bit?
            begin
              {$IFDEF LTSHOW}
              for col:=startcol to startcol+rectWidth-1 do
              {$ELSE}
              for col:=startcol to startcol+AUTO_EXPOSURE_ROI_WIDTH-1 do
              {$ENDIF}
                begin
                  p[3*col] := Min( 255, p[3*col] + BRIGHT_LEVEL );
                  p[3*col+1] := Min( 255, p[3*col+1] + BRIGHT_LEVEL );
                  p[3*col+2] := Min( 255, p[3*col+2] + BRIGHT_LEVEL );
                end;
            end;
        end;
    end;
end;
*)
procedure TImageForm.WMEraseBkgnd(var m : TWMEraseBkgnd);
begin
  m.Result := LRESULT(False);
  //m.Result := LRESULT(True);
end;

procedure TImageForm.ShowImageMouseDown(Sender: TObject;
				Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if ( ClientWidth < ImagePanel.Width ) OR ( ClientHeight < ImagePanel.Height ) then
    begin
      StartX := X;
      StartY := Y;
      XPosMin := 0;
      XPosMax := ImagePanel.Width - ClientWidth;
      if XPosMax < 0 then
        XPosMax := 0;
      YPosMin := 0;
      YPosMax := ImagePanel.Height - ClientHeight;
      if YPosMax < 0 then
        YPosMax := 0;
      XPos := HorzScrollBar.Position;
      YPos := VertScrollBar.Position;
      ScrollAllow := True;
      //ShowImage.Cursor := crHandPoint;
    end;
end;

procedure TImageForm.ShowImageMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  if ScrollAllow then
    begin
      // allow fast speed
      ShowImage.Picture.Bitmap.IgnorePalette := true;
      DeltaX := StartX - X;
      DeltaY := StartY - Y;
      if XPosMin <> XPosMax then  // Can be scroll on X direction.
        begin
          XPos := XPos + DeltaX;  // Adjust XPos by DeltaX
          {
            If the new position is still in valid range.
          }
          if ( XPos>=XPosMin )  AND ( XPos<=XPosMax ) then
            HorzScrollBar.Position := XPos;
        end;
      if YPosMin <> YPosMax then  // Can be scroll on Y direction.
        begin
          YPos := YPos + DeltaY;  // Adjust YPos by DeltaY
          if ( YPos>=YPosMin )  AND ( YPos<=YPosMax ) then
            VertScrollBar.Position := YPos;
        end;

      ShowImage.Picture.Bitmap.IgnorePalette := false;
    end;
end;

procedure TImageForm.ShowImageMouseUp(Sender: TObject;
                                      Button: TMouseButton;
                                      Shift: TShiftState; X, Y: Integer);
begin
  if  ScrollAllow then
    begin
      ScrollAllow := False;
      //ShowImage.Cursor := crDefault;
    end;
end;

procedure TImageForm.FormCreate(Sender: TObject);
begin
  Fitbmp := TBitMap.Create;
  ShowVideo := False;
  {$IFDEF LTSHOW}
//  MarkStartX := 50;
//  MarkStartY := 50;
//  MarkWidth := 5;
  {$ENDIF}
  ScrollAllow := False;
  ImagePanel.Width  := 160;
  ImagePanel.Height := 120;
  //ClientWidth  := PRESET_SHOW_IMAGE_WIDTH;
  //ClientHeight := PRESET_SHOW_IMAGE_HEIGHT;
  DoubleBuffered := True;
  frameRate := 0;
  AlwaysFitWindow := false;
end;

procedure TImageForm.SetDeviceType(ImageSize:TImageSize );
var
  rowSize, colSize : integer;
begin
  RowSize := ImageSize.RowNum;
  colSize := Imagesize.ColumnNum;
  ImagePanel.Width  := rowSize div 2;
  ImagePanel.Height := colSize div 2;
end;

procedure TImageForm.FormDestroy(Sender: TObject);
begin
 Fitbmp.Free;
end;

procedure TImageForm.FormSizeChanged();
begin
   Fitbmp.Width := ClientWidth;
   Fitbmp.Height := ClientHeight;
end;

procedure TImageForm.FormResize(Sender: TObject);
begin
  FormSizeChanged;
end;

procedure TImageForm.FormCanResize(Sender: TObject; var NewWidth,
  NewHeight: Integer; var Resize: Boolean);
var
  NewClientWidth, NewClientHeight : integer;
  HBarWidth, VBarWidth: integer;
begin
  if ControlReSize = False then
    begin
      ReSize := True;
      Exit;
    end;

  NewClientWidth := NewWidth ; //- ExtraWidth;
  NewClientHeight:= NewHeight; //- ExtraHeight;
  HBarWidth := 26 ;
  VBarWidth := 44 ;

  Resize := True;
  if AlwaysFitWindow = False then
    begin
      if ( (NewClientWidth < ClientWidth) AND (NewClientHeight < ClientHeight) ) then
        Exit;
      if     ( NewClientWidth >  (ImagePanel.Width+HBarWidth) )
          OR ( NewClientHeight > (ImagePanel.Height+VBarWidth) )
        then
          begin
            Resize := False;
        end;
    end
  else //AlwaysFitWindow = True
    begin
    end;
end;

procedure TImageForm.FormShow(Sender: TObject);
begin
  ControlReSize := True;
  ShowVideo := True;
  NeedReStartFrameRateCaculation := True;
  PaintInfo.HasPaintCmd := False;
end;

procedure TImageForm.FormHide(Sender: TObject);
begin
  ShowVideo := False;
end;

procedure TImageForm.ShowImageDblClick(Sender: TObject);
begin
{
  if AlwaysFitWindow then
    begin
      if BorderStyle = bsSizeToolWin then // Window style
        begin
          //PostMessage( Handle, wm_SysCommand, SC_MAXIMIZE, 0 );
          SavedClientRect.Left :=  Left;
          SavedClientRect.Top := Top;
          SavedClientRect.Right := Left + Width;
          SavedClientRect.Bottom := Top + Height;
          // Set it to Full Screen.
          BorderStyle := bsNone;
          Left := 0;
          Top := 0;
          ClientWidth := Screen.Width;
          ClientHeight := Screen.Height;
          //SetToNoBorder := True;
        end
      else   // bsNone...so now it's full screen.
        begin
          // Set it back.
          BorderStyle := bsSizeToolWin;
          Left := SavedClientRect.Left;
          Top := SavedClientRect.Top;
          Width := SavedClientRect.Right - Left;
          Height := SavedClientRect.Bottom - Top;
        end;
    end;
}
end;

// =========================================================================
procedure TImageForm.SetShowMode( IsFitInWindow : Boolean; IsShowAutoExMark : Boolean );
begin
    AlwaysFitWindow := IsFitInWindow;
    ShowAutoEXMark := IsShowAutoEXMark;
end;

procedure TImageForm.DisplayImage( localBitMap : pBitMap );
//var
//  ImageRect : TRect;
begin
  if ShowVideo then
    begin
      if AlwaysFitWindow = False then //Same as capture resolution
        begin
//          if ShowAutoEXMark then
//            SetAutoExposureMark( localBitMap^ );
//          if flagCSArea then
//            SetCCDSpectrumAreaMark(localBitMap^);

          {
             JTZ: Note that it doesn't copy bitmap to video memory, but
             put WM_PAINT MSG there, so the localBitMap^ should NOT be
             altered until the WM_PAINT message is processed.
             In this show mode, the ImagePanel should hold 1:1 resolution
             of the BMP image...while the the windows size is controlled
             by user.

             Basically, we have the following:
              Windows Size (ClientWidth/Height)
              ImagePanel Size (fit to 1:1 BMP Size or Windows Size)
              BMP size

             While any time the resolution changes (BMP Size changes), we
             should:
             *. In 1:1 resolution mode:
             1). ImagePanel Size will change to BMP size
             2). Windows Size may change if BMP size is smaller than the
                 pre-set size.
             *. In "Fit" mode
             1). ImagePanel Size will always be current window size
             2). Windows size is the current.
          }
          ShowImage.Picture.Bitmap := localBitMap^ ;
        end
      else
        begin
          {
            In case of AlwaysFitWindow, the ImagePanel size is set to the
            Window's current size(ClientWidth/ClientHeight)...and Bitmap
            is always to fit it.
          }
          (*
          ImageRect.Left := 0;
          ImageRect.Top := 0;
          ImageRect.Right := ClientWidth;
          ImageRect.Bottom:= ClientHeight;
          ImagePanel.Width := ClientWidth;
          ImagePanel.Height := ClientHeight;
          *)
//          if ShowAutoEXMark then
//             SetAutoExposureMark( localBitMap^ );
//          if flagCSArea then
//             SetCCDSpectrumAreaMark(localBitMap^);

          PaintInfo.HasPaintCmd := True;
          PaintInfo.BmpPointer := localBitMap;
          PaintInfo.ImageRect.Left := 0;
          PaintInfo.ImageRect.Top := 0;
          PaintInfo.ImageRect.Right := ClientWidth;
          PaintInfo.ImageRect.Bottom:= ClientHeight;
          PostMessage( Handle, WM_USER_PAINT, 0 , 0 );
          //Fitbmp.Canvas.StretchDraw( ImageRect, localBitMap^ );
          //ShowImage.Picture.Bitmap := Fitbmp;
        end;
    end;
end;

procedure TImageForm.PaintImageFitWindow( var Msg : TMessage );
begin
  if TMessage(Msg).Msg = WM_USER_PAINT then
    begin
      if PaintInfo.HasPaintCmd then
        begin
          PaintInfo.HasPaintCmd := False;
          ImagePanel.Width := PaintInfo.ImageRect.Right;
          ImagePanel.Height := PaintInfo.ImageRect.Bottom;
          Fitbmp.Canvas.StretchDraw( PaintInfo.ImageRect, PaintInfo.BmpPointer^ );
          ShowImage.Picture.Bitmap := Fitbmp;
        end;
    end;
end;
(*
procedure TImageForm.SetStatusPaneInfo( ImageProperty : PProcessedDataProperty ;
                                        IsSelectedCamera : Boolean );
var
  TimeEllapse : integer;
  TitleStr : String;
begin
  //if ShowVideo = False then
  //  Exit;
  if NeedReStartFrameRateCaculation then
    begin
      AccumulateFrames := 0;
      NeedReStartFrameRateCaculation := False;
    end;

  if IsSelectedCamera then
    TitleStr := 'Cam[' +  IntToStr( ImageProperty.CameraID )+ '] '
  else
    TitleStr := 'Cam ' +  IntToStr( ImageProperty.CameraID )+ '  ';
  TitleStr := TitleStr + IntToStr( ImageProperty.Column ) + 'x' + IntToStr( ImageProperty.Row ) + ' ';
  if ImageProperty.Bin = 1 then
     TitleStr := TitleStr + '(1:2) '
  else if ImageProperty.Bin = 2 then
     TitleStr := TitleStr + '(1:3) '
  else if ImageProperty.Bin = 3 then
     TitleStr := TitleStr + '(1:4) ';
  TitleStr := TitleStr +  '(' + IntToStr( ImageProperty.XStart ) + ',' + IntToStr( ImageProperty.YStart ) + ') ';
  TitleStr := TitleStr + IntToStr( ImageProperty.ExposureTime * 50 ) + ' ';
  {
  TitleStr := TitleStr + '(' + IntToStr( ImageProperty.RedGain ) +
                       ',' + IntToStr( ImageProperty.GreenGain ) +
                       ',' + IntToStr( ImageProperty.BlueGain ) + ') ';
  }
  TitleStr := TitleStr + '(' + IntToStr( ImageProperty.RedGain ) + ',' +
                             IntToStr( ImageProperty.GreenGain ) + ',' +
                             IntToStr( ImageProperty.BlueGain ) + ')';
  TitleStr := TitleStr + IntToStr( ImageProperty.TimeStamp ) + ' ';
  TitleStr := TitleStr + IntToStr( ImageProperty.TriggerOccurred ) + ' ' ;
  TitleStr := TitleStr + IntToStr( ImageProperty.TriggerEventCount ) + ' ' ;
  TitleStr := TitleStr + IntToStr( ImageProperty.FrameTime ) + ' ' ;
  TitleStr := TitleStr + IntToStr( ImageProperty.CCDFrequency ) + ' ' ;

  {
  CameraIDPane.Caption := IntToStr( ImageProperty.CameraID );
  if IsSelectedCamera then
    CameraIDPane.Font.Color := clRED
  else
    CameraIDPane.Font.Color := clBLACK;
  ResolutionPane.Caption := IntToStr( ImageProperty.Column ) + 'x' +
                            IntToStr( ImageProperty.Row );
  if ImageProperty.Bin = 1 then
     ResolutionPane.Caption := ResolutionPane.Caption + '(1:2)' ;
  XYStartPane.Caption := '(' + IntToStr( ImageProperty.XStart ) +
                         ',' + IntToStr( ImageProperty.YStart ) + ')';
  ExposurePane.Caption := IntToStr( ImageProperty.ExposureTime * 50 );
  GainsPane.Caption := '(' + IntToStr( ImageProperty.RedGain ) +
                       ',' + IntToStr( ImageProperty.GreenGain ) +
                       ',' + IntToStr( ImageProperty.BlueGain ) + ')';
  TimeStampPane.Caption := IntToStr( ImageProperty.TimeStamp );
  TriggerOccurPane.Caption := IntToStr( ImageProperty.TriggerOccurred );
  TriggerEventPane.Caption := IntToStr( ImageProperty.TriggerEventCount );
  }
  if ( (ImageProperty.ExposureTime div 20 ) < 3000 ) then // Exposure time is less than 3000ms
    begin
      Inc( AccumulateFrames );
      if AccumulateFrames = 1 then
        startTimeTick := ImageProperty.TimeStamp;
      if AccumulateFrames = CAL_FRAMES then
        begin
          AccumulateFrames := 0;
          endTimeTick := ImageProperty.TimeStamp;
          if endTimeTick < startTimeTick then
            TimeEllapse := endTimeTick + $10000 - startTimeTick
          else
            TimeEllapse := endTimeTick - startTimeTick;
          {
            JTZ: As TimeStamp is got (on DSP) at the end of a frame,
            so the real frame between endTimeTick and startTimeTick
            is really (CAL_FRAMES-1).
          }
          frameRate := ( (CAL_FRAMES-1) * 1000 + (TimeEllapse div 2) ) div TimeEllapse;
          //FrameRatePane.Caption := IntToStr(frameRate) + 'fps';
        end;
    end
  else
    begin
      frameRate := 0;
    end;
  TitleStr := TitleStr + IntToStr(frameRate) + 'fps';
  TForm(Self).Caption := TitleStr;
end;
*)


end.
