unit myDatatypes;

interface
uses
  Graphics;
const
  MaxPolyDegree = 20;
  WAVELENGTH_CALC_POLY = 4;
  MAX_CCD_POINTs = 2448;

type

  pBitMap = ^TBitmap;
  
  //-------------------------------------------------
  //---- About PolyFitting coeffs
  //1) polydegree, the items no of function,
  //    e.g. if a polyfunction = A0+A1*x^1,the polydegree is 2:A0,A1
  //    and if a polyfunction= A0+A1*x^1+A2*x^2+A3*x^3,the polydegree is 4
  //2) PolyCoeff-defined as TDegreeVector,indexing from 1
  //    so, PolyCoeff[0] holds A0,
  //        PolyCoeff[1] holds A1, and so on
  //-------------------------------------------------
  TPolyDegree = 0..MaxPolyDegree-1;
  TDegreeVector = Array[TPolyDegree] of double; // from polyfit.pas

  PWaveCalCoeff = ^TWaveCalCoeff;
  TWaveCalCoeff = array[0..WAVELENGTH_CALC_POLY-1] of double;

  PWavelengthCalibPara = ^TWavelengthCalibPara;
  TWavelengthCalibPara = record
    existed      : Boolean;
    WCalCoeff	   : TWaveCalCoeff;		//Pixel To Wavlength Calibration Coeffs;
    PCalCoeff	   : TWaveCalCoeff; 	//Wavelength To pixel Calibration Coeffs;
  end;

  PETCCalibPara = ^TETCCalibPara;
  TETCCalibPara = record
    Existed		  :boolean;
    slope		    :double;       //expected intensity line slope.
    PolyDegree  :tPolyDegree;
    PolyCoeff	  :TDegreeVector;
  end;

  PRRCCalibPara = ^TRRCCalibPara;
  TRRCCalibPara = record
    Existed		:boolean;
    CalibData	:array[0..MAX_CCD_POINTS-1] of double;
  end;

  eLampIntenUnit = (uWcmnm=0,mWcmnm,Wmnm,maxIntenUnitCount);
  PAICCalibPara = ^TAICCalibPara;
  TAICCalibPara = record
    Existed   : Boolean;
    IntenUnit : eLampIntenUnit;
    CalibData : array[0..Max_CCD_POINTS-1] of double;
  end;

  PSpectrometerProperty = ^TSpectrometerProperty;
  TSpectrometerProperty = record
    MaxPixelPoints:Integer;     // pixel point range: 0..MaxPixelPoints-1     
    //MinExpTime:integer;
    WCalibPara  : TWavelengthCalibPara;  
    ETCCalibPara: TETCCalibPara;
    RRCCalibPara: TRRCCalibPara;
    AICCalibPara: TAICCalibPara;
  end;

  TImageSize = record
    RowNum    : integer;
    ColumnNum : integer;
  end;

  pCCDDataArray = ^TCCDDataArray;
  TCCDDataArray = array[0..MAX_CCD_POINTS-1] of double;
  
  POutputData = ^TOutputData;
  TOutputData = record
    CCDData     :TCCDDataArray;  //0 based
    CalibData   :TCCDDataArray;
    DarkSubData :TCCDDataArray; 
    ToverRData  :TCCDDataArray;
    AbsorbData  :TCCDDataArray;
    AbsIntenData:TCCDDataArray;
  end;


var

    Deviceproperty : record
      DeviceType      : integer;
      MaxCCDSize      : TImageSize;
      SliceNum        : integer;
    end;

    SpectroProperty :TSpectrometerProperty;

    OutputData : TOutputData;
    FrameData : array[0..1039] of array[0..1391] of Word;
implementation

end.
